/**
 ** KSplash2k for KSplash/ML
 ** (An Example ThemeEngine)
 ** Copyright (c) 2002, Flaming Sword Productions
 ** All Rights Reserved
 **
 ** $Id: RotWidget.cpp,v 1.6 2002/05/21 02:07:54 brian Exp $
 **/
#include	<kpixmap.h>
#include	<kpixmapeffect.h>
#include	<qlabel.h>
#include	<qwidget.h>
#include	<qpainter.h>
#include	<kpixmap.h>
#include	<kdialogbase.h>
#include	"RotWidget.h"

/**
 ** Author's note: This is a really _crude_ piece of code.  I have made no effort
 ** to optimize this (via direct blt's, double-buffering, or any other method) at
 ** all, so on some X displays, this will really stink.  If someone wishes to send
 ** me a "better" version of this code, I would be happy to re-release this example
 ** ThemeEngine with your name in the spotlight.  If not, don't bother complaining
 ** about the way this widget looks to me.  It's not worth the time it takes to get 
 ** it right (and it looks good enough on fast computers).
 **
 ** Sorry fer the hard-nozed attitude.  Perhaps I'm just in a foul mood this evening.
 **/

/**
 ** RotWidget::RotWidget(): Initialize our widget with a set of colors.
 **/
		RotWidget::RotWidget( QWidget *parent, const QColor& c1, const QColor& c2, int sp )
		:QWidget( parent ),
		color1 (c1), color2 (c2), step (0), speed (sp) {

	if( (speed <= 0) || (speed > 20) ) speed = 1;

	setFixedHeight( 6 );

	for( int i = 0; i <= width(); i++ ) {
		_preparePixmap( i );
	}

	stepTimer = new QTimer( this );
	connect(stepTimer, SIGNAL(timeout()), this, SLOT(stepEvent()));
	stepTimer->start( 2 );
}

/**
 ** RotWidget::~RotWidget(): Clean up after ourselves.
 **/
		RotWidget::~RotWidget() {
}

/**
 ** RotWidget::stepEvent(): This is called every time our timer signals.
 ** At this point, we will want to do a full refresh, even if we don't think
 ** we need to.
 **/
void		RotWidget::stepEvent() {

	++step;
	if( step > width() ) step = 0;

	repaint( true );
}

/**
 ** RotWidget::paintEvent(): Paint ourselves from whichever current StepMap
 ** we are looking at.
 **/
void		RotWidget::paintEvent( QPaintEvent *pe ) {

	QPainter p;
	p.begin( this );

	if( stepPixmap.at( step ) ) {
		p.drawPixmap( rect(), *(stepPixmap.at( step )) );
	} else {
		p.fillRect( rect(), Qt::black );
	}
	p.end();
}

/**
 ** RotWidget::resizeEvent(): We've gotta repaint all of our gradients now.
 ** Dang.
 **/
void		RotWidget::resizeEvent( QResizeEvent *re ) {

	stepPixmap.clear();
	for( int i = 0; i <= re->size().width(); i++ ) {
		_preparePixmap( i );
	}
}

/**
 ** RotWidget::_preparePixmap(): Prepare a gradiented pixmap for a particular
 ** step.
 **/
void		RotWidget::_preparePixmap( int step ) {

	if( step < 0 ) return;

	/**
	 ** We've gotta explicitly draw our first pixmap.  The rest we will
	 ** bitBlt() from here.
	 **/
	if( step == 0 ) {
		KPixmap tmp( QSize( this->size().width() / 2, this->size().height() ) );
		KPixmap tmp2( QSize( this->size().width() / 2, this->size().height() ) );
		KPixmapEffect::gradient( tmp, color1, color2, KPixmapEffect::HorizontalGradient );
		KPixmapEffect::gradient( tmp2, color2, color1, KPixmapEffect::HorizontalGradient );
		KPixmap *px = new KPixmap( size() );
		QPainter p;
		p.begin( px );
		p.drawPixmap( 0, 0, tmp );
		p.drawPixmap( size().width()/2, 0, tmp2 );
		p.end();
		stepPixmap.append( px );

	/**
	 ** bitBlt() the last image by one pixel to make this work.
	 **/
	} else if( stepPixmap.at( step-1 ) ) {

		QPixmap *prev = stepPixmap.at( step-1 );
		QPixmap next( size() );

		/** 
		 ** convert
		 ** prev = "[------------]"
		 ** to
		 ** next = "------------]["
		 **/
		bitBlt( &next, 0, 0, prev, 1, 0, prev->width()-speed, prev->height() );
		bitBlt( &next, width()-speed, 0, prev, 0, 0, speed, prev->height() );

		KPixmap *n = new KPixmap( next );
		stepPixmap.append( n );
	}
}
