/**
 ** KSplash2k for KSplash/ML
 ** (An Example ThemeEngine)
 ** Copyright (c) 2002, Flaming Sword Productions
 ** All Rights Reserved
 **
 ** $Id: Theme2k.cpp,v 1.7 2002/06/11 16:16:01 brian Exp $
 **/
#include	<qlabel.h>
#include	<qwidget.h>
#include	<kapp.h>
#include	<kstddirs.h>
#include	<klocale.h>
#include	<kdialogbase.h>
#if ( QT_VERSION > 300 )
#include	<qdesktopwidget.h>
#else
#include	<kdesktopwidget.h>
#endif
#include	"ThemeEngine.h"
#include	"Theme2k.h"

/**
 ** Cfg2k::Cfg2k(): Prepare a configuration dialog.  I don't think
 ** we'll be needing this...
 **/
		Cfg2k::Cfg2k( KConfig * ) {
}

/**
 ** Theme2k::Theme2k(): The constructor should perform all of the tasks
 ** necessary to run your Theme.
 **/
		Theme2k::Theme2k( QWidget *parent, ObjKsTheme *theme )
		:ThemeEngine( parent, theme ) {

	_readSettings();
	_initUi();
}

/**
 ** Theme2k::_initUi(): This is a method I usually use to prepare the
 ** user interface.  It's not required for you to do things _exactly_
 ** like this.
 **/
void		Theme2k::_initUi() {

	QVBox *vbox = new QVBox( this );
	vbox->setFrameShape( QFrame::WinPanel );
	vbox->setFrameShadow( QFrame::Raised );

	QHBox *labelBox = new QHBox( vbox );
	labelBox->setPalette( oTBgColor );
	labelBox->setMargin( 1 );
	QLabel *lbl = new QLabel( oWndTitle, labelBox );
	lbl->setFont( QFont( "Arial", 12, QFont::Bold ) );
	lbl->setPaletteForegroundColor( oTFgColor );

	QLabel *logo = new QLabel( vbox );
	logo->setPalette( Qt::white );

	QString px( locate( "data", gTheme->themeDir() + "/Logo.png" ) );
	if( !px.isNull() ) {
		QPixmap pix( locate( "data", gTheme->themeDir() + "/Logo.png" ) );
		logo->setPixmap( pix );
	} else {
		logo->setText( "<B>KDE</B>2000" );
		logo->setAlignment( AlignCenter|AlignVCenter );
	}

	gRotator = new RotWidget( vbox, oRotColor1, oRotColor2, oRotSpeed );

	QHBox *hbox = new QHBox( vbox );
	labelBox->setSpacing( 4 );
	labelBox->setMargin( 4 );

	gText = new QLabel( hbox );
	gText->setText( oWndTitle );
	gText->setFixedHeight( 48 );

	setFixedSize( vbox->sizeHint() );

	QRect rect = _desktopGeometry();

	move( rect.x() + (rect.width() - size().width())/2,
		rect.y() + (rect.height() - size().height())/2 );
}

/**
 ** Theme2k::_desktopGeometry(): I use this function to determine the
 ** size of the screen.  This function _only_ returns a QRect representing
 ** the _PRIMARY_ XINERAMA display [or the whole screen, if XINERAMA is
 ** disabled].  If you want to center your theme across all screens (not
 ** really recommended, but you're the boss), then you probably want to
 ** just call KApplication::desktop().geometry() to get this data.
 **/
QRect           Theme2k::_desktopGeometry() {

#ifdef  QT_NO_XINERAMA
	QRect rect = KApplication::desktop().geometry();
#else
#if (QT_VERSION > 300)
	QDesktopWidget *desktop = (QDesktopWidget *)KApplication::desktop();
	QRect rect = desktop->screenGeometry( desktop->primaryScreen() );
#else
	KDesktopWidget *desktop = KApplication::desktop();
	QRect rect = desktop->screenGeometry( desktop->primaryScreen() );
#endif
#endif

	return( rect );
}

/**
 ** Theme2k::_readSettings(): Determine what the current Theme wishes to do.
 ** Since this is only an example Engine, I will not be making this too 
 ** detailed.  This should show you all of the steps necessary to read in the
 ** customized settings from Theme.rc files which use your ThemeEngine.
 **/
void		Theme2k::_readSettings() {

	if( !gTheme ) return;

	// TRAP: Can't operate on a NULL object.
	KConfig *cfg = gTheme->themeConfig();
	if( !cfg ) return;
	
	if( !cfg->hasGroup( QString("KSplash/ML Theme: %1").arg(gTheme->theme()) ) ) return;
	cfg->setGroup( QString("KSplash/ML Theme: %1").arg(gTheme->theme()) );

	QColor DefaultTBgColor( Qt::darkBlue );
	QColor DefaultTFgColor( Qt::white );

	oTBgColor = cfg->readColorEntry( "Title Background Color", &DefaultTBgColor );
	oTFgColor = cfg->readColorEntry( "Title Foreground Color", &DefaultTFgColor );

	QColor DefaultRot1( Qt::darkBlue );
	QColor DefaultRot2( Qt::cyan );
	oRotColor1 = cfg->readColorEntry( "Rotator Color 1", &DefaultRot1 );
	oRotColor2 = cfg->readColorEntry( "Rotator Color 2", &DefaultRot2 );

	oRotSpeed = cfg->readNumEntry( "Rotator Speed", 20 );

	oWndTitle = cfg->readEntry( "Window Title", i18n("Please wait...") );

	oLogoFile = cfg->readEntry( "Logo File", QString::null );
}
